/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guichangeaccountname.h"
#include "aqmoney2/account.h"
#include "openhbci2/pointer.h"

#include <qlineedit.h>
/**
  *class to change name of account 
  *
  *@param main
  *@param parent the parent dialog of this widget
  *@param name widgets title
  *@param hobit
  */
GuiChangeAccountName::GuiChangeAccountName(QWidget *parent, GuiMain* guimain, const char *name, HoBIT* hobit)
    :GuiChangeAccountNameBase(parent, name)
{
  _main = guimain;
  _hobit = hobit;
  qDebug("Constructor GuiChangeAccountName");
}


/**destructor*/
GuiChangeAccountName::~GuiChangeAccountName(){
  qDebug("Destructor GuiChangeAccountName");
}


/**
*returns to/shows last visible Widget
*/
void GuiChangeAccountName::slotBack()
{
  _main->slotShowLastVisibleWidget();
}

/**
*function to rename account and refresh the current mainwindow
*/
void GuiChangeAccountName::accept()
{
  _hobit->changeAccountName(lineEditAccountId->text().ascii(), lineEditAccountName->text().ascii());
  _main->slotShowLastVisibleWidget();
  _main->slotRefresh();
}

 /** Called on show. Sets current account.
 */
 void GuiChangeAccountName::showEvent ( QShowEvent * ){
  Pointer<Account> account;
  QString accountId = _hobit->getCurrentAccount();
  lineEditAccountId->setText(accountId);

  if (! accountId.isEmpty()){
    account = _hobit->findAccount(accountId);
    qDebug("Editing name of account: %s", accountId.latin1());
  }
  else{
    qDebug("GuiChangeAccountName::showEvent: Can not edit an account with empty account id.");
    return; 
  }  
  
  if (account.isValid())
  {
    QString accountName(account.ref().accountName().c_str());
    if (accountName.isEmpty()) accountName = "";
    lineEditAccountName->setText(accountName);
  } 
 }

/** set hobit object
 */
  void GuiChangeAccountName::setHoBIT(HoBIT* hobit){
   _hobit = hobit;
  }