/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guidialin.h"

#include <qprocess.h>
#include<qmessagebox.h>

typedef enum {
 PROCESS_UNKNOWN,
 PROCESS_MODEM,
 PROCESS_ISDN,
 PROCESS_DSL, 
 PROCESS_LAN, 
 PROCESS_WLAN, 
 PROCESS_GPRS 
}ProcessTypes;

GuiDialIn::GuiDialIn(QWidget* parent, const char* name)
 : GuiDialInBase(parent, name)
{
 qDebug("Constructor GuiDialIn");
 _Process = NULL;
 _ProcessType = PROCESS_UNKNOWN;
 _Process = new QProcess(this);  
 connect(_Process, SIGNAL(processExited()), this, SLOT(slotProcessExited()));
}


GuiDialIn::~GuiDialIn()
{
 if(_Process)
   delete _Process;
 qDebug("Destructor GuiDialIn");
}

/** Dial in via Modem.
*/
void GuiDialIn::slotModem(){
   
   if(_Process)
   {   
     _Process->kill();
     _Process->clearArguments();
     _ProcessType = PROCESS_MODEM;
     _Process->addArgument("kppp");
     _Process->start();
     qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
   }
}

/** Dial in via ISDN.
*/
void GuiDialIn::slotISDN(){
   if(_Process)
   {   
     _Process->kill();
     _Process->clearArguments();
     _ProcessType = PROCESS_ISDN;
     _Process->addArgument("sudo"); 
     _Process->addArgument("/etc/init.d/isdnutils");
     _Process->addArgument("start"); 
     _Process->start();
     qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
   }  
}

/** Dial in via DSL.
*/
void GuiDialIn::slotDSL(){
   if(_Process)
   {   
     _Process->kill();
     _Process->clearArguments();
     _ProcessType = PROCESS_DSL;
     _Process->addArgument("sudo");
     _Process->addArgument("pon");
     _Process->addArgument("dsl-provider"); 
     _Process->start();
     qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
   }  
}

/** Call browser to activate router via http.
*/
void GuiDialIn::slotLAN(){
   if(_Process)
   {   
     _Process->kill();
     _Process->clearArguments();
     _ProcessType = PROCESS_LAN;
     _Process->addArgument("firefox");
     _Process->start();
     qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
   }  
}

/** Call browser to activate router via http.
*/
void GuiDialIn::slotWLAN(){
   if(_Process)
   {   
     _Process->kill();
     _Process->clearArguments();
     _ProcessType = PROCESS_WLAN;
     QMessageBox::information(this, tr("Not implemented"),
      tr("Sorry, this feature is not yet implemented"));
   }
}

/** Dial in via GPRS.
*/
void GuiDialIn::slotGPRS(){
   if(_Process)
   {   
     _Process->kill();
     _Process->clearArguments();
     _ProcessType = PROCESS_GPRS;
     QMessageBox::information(this, tr("Not implemented"),
      tr("Sorry, this feature is not yet implemented"));
   }   
}

/** Emits signal finished()
*/
void GuiDialIn::slotFinished(){
  emit finished();
}

/** Called when process has finished. Issues successor failure messages.
*/
void GuiDialIn::slotProcessExited(){
  QString msg;
 if(!_Process)
   return; 
 if(_Process->normalExit())
    switch (_ProcessType){
      case PROCESS_MODEM: msg = tr("The dialing program has been terminated.");break;
      case PROCESS_ISDN: msg = tr("The ISDN system has been started.");break;
      case PROCESS_DSL: msg = tr("The DSL connection has been started.");break;
      case PROCESS_LAN: msg = tr("The browser has been terminated.");break;
    //  case PROCESS_WLAN: msg = ".";break;
    //  case PROCESS_GPRS: msg = "";break;
      default: msg= tr("The dial in process has terminated.");
    }
 else msg = tr("The dial in process has failed.");
    
 QMessageBox::information(this, tr("Information"), msg); 
}

