/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guifindmedium.h"

#include <stdlib.h>

#include <qpushbutton.h>
#include <qfile.h>
#include <qstringlist.h>
#include <qradiobutton.h>
#include <qregexp.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qdir.h>
#include <qcursor.h>
#include <qprocess.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qframe.h>
#include <qcursor.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qlistbox.h>

GuiFindMedium::GuiFindMedium(QWidget * parent)
 : GuiFindMediumBase(parent)
{
 connect(&_Timer, SIGNAL(timeout()), this, SLOT(slotTimerDone()));
 slotTryAgain(); //reset
 if (BUILD_FOR_MAC_OSX){ 
   frameLinux->hide();
 }
 else {
  frameOSX->hide();
 }
 frameBankList->hide();
 resize(width(), 300);
}

GuiFindMedium::~GuiFindMedium()
{
 qDebug("Destructor GuiFindMedium finished");
}

/** Read property of StorageType m_Storage. 
*/
const StorageType& GuiFindMedium::getStorage()const{
	return _Storage;
}

/** Write property of StorageType m_Storage. 
*/
void GuiFindMedium::setStorage( const StorageType& _newVal){
	_Storage = _newVal;
}

/** Runs the whole self-finding process for a storage device
*/
void GuiFindMedium::slotTimerDone(){  
   //do not interfere with running processes
   if(_SkipTimer)
     return;

   //count down after Medium has been found
   if(_NumberOfBanks != 0 ){ 
         buttonOk->setEnabled(true);
        if(_NumberOfBanks == 1)//only one bank, or new medium, proceed automatically
          _TimeToShow --;
        if(_TimeToShow <= 0) 
          accept(); // proceed after _TimeToShow
   }

   if(_MediumFound && _IsWritable) //return after Medium has been found
     return; 

   //Start searching for writable medium

   QString writeProtectMsg = 
     tr("The storage device seems to be write protected.")+ "<br>"
    + tr("Will retry writing. Please wait a little longer.") + "<br>"
    + tr("If this does not succeed, remove device, make it writable and try again.");

   //Search for Medium
   if(_Storage == STORAGE_UNKNOWN){ // search for device
     if (BUILD_FOR_MAC_OSX)
       searchForVolumeOSX();
     else if(USE_UDRW_FEATURES && !(radioButtonUsb->isChecked() || radioButtonFloppy->isChecked()))
       searchForUdrwDevice();
     else if(radioButtonFloppy->isChecked())
       searchForFloppy();
     else if(radioButtonUsb->isChecked()){
       searchForUsbDevice();
     }
     return;
   }//end if STORAGE_UNKNOWN
   else 
   { // device found or selected, wait for /proc/mounts to be updated
     if((_Storage == STORAGE_USB) ){
        if(_MountPoint.isEmpty())
          _MountPoint = findMountPoint(); // this will not work immediately, 
        if(USE_LIVE_CD_FEATURES && _MountPoint.isEmpty())
           runProcess("sudo mount " +_DeviceName); //try to mount manually

        setStatusText(
            tr("New USB device found. Checking for writability. This will take a few seconds.<br> Please wait."),
            yellow);
        if(_MountPoint.length() > 0){ 
          unsetCursor();
          qDebug("USB device found: %s", _MountPoint.latin1());
          _IsWritable = canWriteTestFile();
          if(_IsWritable){
            setStatusText(tr("USB device is writable. Please proceed."), green);
            _MediumFound = true;
         }//end mountpoint found
       }
      }//end USB

      else if(_Storage == STORAGE_FLOPPY){ 
 //       _MountPoint = findMountPoint("/proc/mounts", _DeviceName);
        if(_MountPoint.length() > 0){ 
           _IsWritable = canWriteTestFile();
           if(_IsWritable){
             qDebug("Floppy found: %s", _MountPoint.latin1());
             setStatusText(tr("A floppy disc has been found. Please proceed."), green);
             _MediumFound = true;
           }
        }//end length > 0
      }//end floppy  

      else if(_Storage == STORAGE_OSX){
         setStatusText(
        tr("New storage device found. Checking for writability. This will take a few seconds.<br> Please wait."),
          yellow);
	 _IsWritable = canWriteTestFile();
         if(_IsWritable){
           qDebug("Storage device found: %s", _MountPoint.latin1());
           setStatusText(tr("Storage device is writable. Please proceed."), green);
           _MediumFound = true;
         }
      }//end OSX

      else if(_Storage == STORAGE_MANUALLY){ 
           _IsWritable= canWriteTestFile();
           if(_IsWritable)
             setStatusText(tr("A writable storage device has been found. Please proceed."), green);
           else
             setStatusText(writeProtectMsg, yellow);
      }//end storage manual  

      if((_MountPoint.length() > 0)&&! (_IsWritable )) {//device is not writable
         _IsWritable = canWriteTestFile(); //retry
         _WriteCycles ++;
         if(_WriteCycles > 10){
           qDebug("Error: Device is write protected.");
           setStatusText(writeProtectMsg, red);
         }
      }
   }//end else  device found
   
    //mountpoint found for a writable device, build the bank list
   if((_MountPoint.length() > 0) && _IsWritable){
      if(_NumberOfBanks == 0)
         _NumberOfBanks = buildBankList(); 
      if(_NumberOfBanks == 0)
        buttonOk->setText(tr("Continue anyway!"));
      buttonOk->setEnabled(true); //allow to continue  
      
  }//end mountpoint found 
}

/** Starts the timer loop
*/
void GuiFindMedium::showEvent(QShowEvent * se){
    radioButtonUsb->setChecked(true);
    slotTryAgain();
    _Timer.start(1000);
}


/** Stops the timer loop
*/
void GuiFindMedium::hideEvent(QHideEvent * se){
   _Timer.stop();
}

/** Tries to read from floppy or USB device
*/
void GuiFindMedium::slotSelectionChanged(){
 if(radioButtonFloppy->isChecked()){
   setStatusText(tr("Waiting for a floppy disk to be inserted now."), yellow);
 }
 else if(radioButtonUsb->isChecked()){
   setStatusText(tr("Waiting for a USB device to be plugged in now."), yellow);
 }
 setCursor(Qt::WaitCursor);
 _Storage = STORAGE_UNKNOWN;
 _MountPoint = "";
 _NumberOfBanks = 0;
 frameBankList->hide();
}

/** Searches for a mounted floppy
*/
bool GuiFindMedium::searchForFloppy(){
 if(USE_LIVE_CD_FEATURES){ //auto mount the floppy
   runProcess("cd /mnt/floppy");
 }
 _DeviceName = "/dev/fd0";
 findMountPoint(); //first try the devicename /dev/fd0
 if(_MountPoint.isEmpty()){
   _DeviceName = "floppy";
   findMountPoint(); //there may be other floppy drives, should be mounted on .../floppy/...
 }
 if(!_MountPoint.isEmpty()){//give up searching, look what we have found
    QDir dir(_MountPoint); 
    QStringList files = dir.entryList();
    if(files.count() > 0){
      _Storage = STORAGE_FLOPPY; //do not set _MountPoint here, it is set in slotTimerDone
      return true;
    } 
    else return false;
 }   
 else return false;
}



/** Searches for a new plugged in volume, 
  *  when moneypenny is running on mac osx
  * Changed: 19.01.06  We are using 'df' instead 'ls' here, so we can scan for '/Volumes/'   (AG)
  */
bool GuiFindMedium::searchForVolumeOSX(){

 bool result = false;
 QStringList newList;
 QStringList sections;
 QString newLine, oldLine; //for easy debugging

  // get moneypenny-tmp-dir
  // this is on the ramdisk, when running from CD
  QString homeDir = getenv("HOME"); 
  QString mpHomeDirName = homeDir + "/" + MONEYPENNY_MAIN_DIR;

  QString checkfile = mpHomeDirName + "/volumes_check";

  system(	QString("df") +  
		// + QString(OSX_MOUNT_PATH) + 
		" > " +
		checkfile);

 QFile volumesFile(checkfile);
 if(volumesFile.open(IO_ReadOnly)){
   QTextStream s(&volumesFile);
   while(! s.atEnd()){ //read present partion list and save it in newList
     newLine = s.readLine();
     	newList.push_back(newLine);
   }//end while
   volumesFile.close();
 }//end if 
 
 // remove temporary file
 system(QString("rm " + checkfile));

 //check for changes in list
 if(_PartitionList.count() > 0){
   if(_PartitionList.count() < newList.count()){

	for (uint i = 0; i < newList.count(); i++){
		
		bool isNewDirectory = true;
		QString newDir;
		for (uint j = 0; j <_PartitionList.count(); j++){
			//if (newList[i].startsWith("d")){  // only get directories
				if (_PartitionList[j] == newList[i]) {
					isNewDirectory = false;
				}
				else {
					 newDir = newList[i];
					}
		       //} else isNewDirectory = false;
		}
		if (isNewDirectory){
  			int pos = newDir.findRev("/Volumes/", newDir.length() -2);
			QString gotDir = newDir.remove(0, pos);
    			_MountPoint = gotDir;
                        _MountFlags = "rw";
			qDebug("New Directory found: " + _OsxVolume);	
 			_Storage = STORAGE_OSX;
			result = true;
		}
	}	
   }//end for nNewLine
   else if (_PartitionList.count() == newList.count()){
   //Nothing to do
   }
   else if (_PartitionList.count() > newList.count()){
     setStatusText(tr("A storage medium has been removed. Please insert your storage medium now."),
        yellow);
     _MountPoint = "";
     _IsWritable = false;
   }//end if >

}
 _PartitionList = newList; //save present list
 return result;

}


/** Searches for a USB-device. Processes entries in /proc/partitions
*/
bool GuiFindMedium::searchForUsbDevice(){
 bool result = false;
 bool isNew = false;
 QStringList newList;
 QStringList sections;
 QRegExp rxDevice("[a-z]{3,3}[0-9]{1,1}"); //3 letters and one digits e.g. sda1
 QRegExp rxSplit("[\\s]"); //any whitespace
 QString newLine, oldLine; //for easy debugging
 QFile partitionsFile("/proc/partitions");
 if(partitionsFile.open(IO_ReadOnly)){
   QTextStream s(&partitionsFile);
   while(! s.atEnd()){ //read present partion list and save it in newList
     newLine = s.readLine();
     newList.push_back(newLine);
   }//end while
   partitionsFile.close();
 }//end if 
 //check for changes in list
 if(_PartitionList.count() > 0){
   if(_PartitionList.count() < newList.count()){
      for(uint nNewLine = 0; nNewLine < newList.count(); nNewLine ++){ // run through the whole list and check all entries
       isNew = true;
       newLine = newList[nNewLine];
       for(uint nOldLine = 0; nOldLine < _PartitionList.count(); nOldLine ++){//compare to all old lines
         oldLine = _PartitionList[nOldLine];
         if(newLine == oldLine) 
           isNew = false;
       }//end for nOLdLine
       if(isNew){  
         sections= QStringList::split(rxSplit, newLine);
         if(rxDevice.exactMatch(sections[sections.count() -1])){ //last column contains device name
           _DeviceName = "/dev/" + sections[sections.count() -1]; // this is only sda1 (uba1) at this point
           qDebug("Device name: %s", _DeviceName.latin1());
           _Storage = STORAGE_USB;
           result = true;
         }// end if match 
       }
     }//end for nNewLine
   }// end if <
   else if (_PartitionList.count() == newList.count()){
   //Nothing to do
   }
   else if (_PartitionList.count() > newList.count()){
     setStatusText(tr("A storage medium has been removed. Please insert your storage medium now."), yellow);
     _MountPoint = "";
     _IsWritable = false;
  }//end if >
 }//end if > 0
 _PartitionList = newList; //save present list
 return result;
}

/** Searches the mountpoint in mtab or /proc/mounts.
  * Sets _DeviceName 
  */
QString GuiFindMedium::findMountPoint(){
 QString result, line;
 QStringList entries;
 bool found = false;
 uint n =0;
 QString configFile = "/proc/mounts";
 QFile file(configFile);
  
 if(file.open(IO_ReadOnly)){
   QTextStream s(&file);
   while((! s.atEnd()) && (!found)){
     line = s.readLine();
     if(line.find(_DeviceName, 0, false)>=0){ //search for the name, whereever it is 
       found = true;
       qDebug("GuiFindMedium::findMountPoint: Device name: %s   Device line: %s", _DeviceName.latin1(), line.latin1());
       entries= QStringList::split(" ", line);
       n = entries.count();
       _MountPoint = entries[1];
       result= _MountPoint;
       _MountFlags = entries[3];
      }
   }//end while
   file.close();
 }//end if 
 return result;
}

/** Read property of QString _CryptDirName. 
*/
const QString& GuiFindMedium::getCryptDirName()const{
	return _CryptDirName;
}

/** Read property of QString _MountPoint. 
*/
const QString& GuiFindMedium::getMountPoint()const{
	return _MountPoint;
}

/**
*shows a short help message in a mesagebox.
*/
void GuiFindMedium::slotHelp(){
   QString msg =
   tr("MoneyPenny needs the removable storage medium in order to save data and key files.<br>\
   Finding a USB device is somewhat tricky, because there may be many USB-devices. \
   The easiest way is to look for a freshly plugged in device.<br>\
   Therefore it is important to switch on the computer <b>without</b> the USB device and plug it in, when MoneyPenny asks you to do so.\
   If you have already plugged in your USB device, remove it and plug it in again, after the removal has been detected.<br>\
   A floppy disk will be found, if it has been inserted, regardless of the insertion time. \
   <p>The program will proceed automatically a few seconds after a medium has been found.</p>");
   QMessageBox::information(this, "MoneyPenny Help", msg);
}

/** Read property of bool _IsWritable. 
*/
const bool& GuiFindMedium::isWritable()const{
	return _IsWritable;
}

/** Closes dialog and creates crypt-directory, if _IsWriteable is true. Else repeats search.
*/
void GuiFindMedium::accept(){
 if((_Storage != STORAGE_UNKNOWN) && _IsWritable){
  _CryptDirName = _MountPoint+ "/" + QString(MONEYPENNY_MAIN_DIR);
   QDir dir(_CryptDirName);
  if(!dir.exists())
     dir.mkdir(_CryptDirName, true);
  if(!dir.exists()){
    QMessageBox::critical(this, tr("Critical Error"), tr("Could not create data directory: ") + _CryptDirName);  
     _Timer.stop();
    return;  
  }
  GuiFindMediumBase::accept();
 }
}

 /** Resets the flags
 */
 void GuiFindMedium::slotTryAgain(){
    _Storage = STORAGE_UNKNOWN;
    buttonOk->setEnabled(false);
    buttonOk->setText(tr("Ok"));
    _TimeToShow = 2;
    _NumberOfBanks = 0;
     _IsWritable = false;
    _MkDirStarted = false;
    _MediumFound = false;
    _DeviceName = "";
    _MountPoint = "";
    _MountFlags = "";
    _CryptDirName = "";
    _SkipTimer = false;
    _PartitionList.clear();
    _ResizeRequired = true;
     frameMore->hide();
    _WriteCycles = 0;
    slotSelectionChanged();
 }
 
 /**Opens an Xterm and a file dialog
 */
 void GuiFindMedium::slotSearchManually(){
  _SkipTimer = true;
  unsetCursor();
  QMessageBox::information(this, tr("How To Search"),
  tr("Sorry for not finding your medium. MoneyPenny will open a shell for experts next.\
   <br>Search the name of your usb device in /proc/partitions, mount it to /mnt/usb-moneypenny and exit the shell.\
   <br>Then a file dialog pops up. Select your mountpoint in this dialog and click Ok."));
  if(USE_LIVE_CD_FEATURES)
    system("sudo xterm");
  else 
    system("xterm");
  QString s = QFileDialog::getExistingDirectory(
                 "/mnt",
                 this,
                 "Select your mountpoint",
                 "Choose a directory",
                 TRUE );
  s = s.remove(s.length()-1, 1); // strip trailing slash /               
  _Storage = STORAGE_MANUALLY;
  qDebug("Mountpoint manually selected: %s", s.latin1());
  _MountPoint= s; //take the mountpoint for devicename 
  _MediumFound = true;
  //_IsWritable = true;
  _SkipTimer = false;
  //accept();
 }
 
 /**Shows the extra buttons
 */
 void GuiFindMedium::slotMore(){
    frameMore->show(); 
   _ResizeRequired = true;
 }
 
/** Runs the specified process, spins the event loop and sets the _MediumFound flag, so that timer events are not processed.
*/
void GuiFindMedium::runProcess(const QString & command){
   QCursor oldCursor = cursor();    
   _MediumFound = true; //do not process pending timer events
   setCursor(WaitCursor);
   qDebug("Starting process: %s", command.latin1());
   system(command.latin1());
   _MediumFound = false; 
   setCursor(oldCursor);
}
/** Resizes window height, if required.
*/
void GuiFindMedium::paintEvent(QPaintEvent * pe){
   if(_ResizeRequired){
     //now that all components are in their right visibilty state, resize and reposition the widget.
     int newHeight = buttonOk->y() + buttonOk->height() + layout()->spacing();
     int newY = (qApp->desktop()->height() - newHeight) /2;
     int newX = (qApp->desktop()->width() - width()) /2;
     setGeometry(newX,newY, width(), newHeight);
     _ResizeRequired = false;
   }  
}

/** If more than one bank is available, shows the bank List.
*  Returns number of banks. Returns -1, if medium is new and no moneypenny_data dir exists.
*/
int GuiFindMedium::buildBankList(){
  if(_MountPoint.isEmpty())
   return 0;
  QDir banksDir = QDir(_MountPoint + "/"+ MONEYPENNY_MAIN_DIR);
  if(!banksDir.exists()) //this is a completely new medium
   return -1; 
  QStringList bankFiles = banksDir.entryList("*" + QString(MONEYPENNY_ENCRYPTED_HBCI_CONF_FILE));
  
  //list all the banknames = prefixes to moneypenny.bf
  listBoxBanks->clear();
  QStringList::iterator itBank = bankFiles.begin();
  while( itBank != bankFiles.end()){
    QString bankName = (*itBank).remove("_" + QString(MONEYPENNY_ENCRYPTED_HBCI_CONF_FILE));
    if(!bankName.isEmpty()){
      if((bankName != MONEYPENNY_ENCRYPTED_HBCI_CONF_FILE)||(bankFiles.count() ==1) ) //show old file only, if no others are available
        listBoxBanks->insertItem(bankName);
    }  
    itBank ++;
  }
  
  if(listBoxBanks->count() <= 1)
    frameBankList->hide();
  else {
    frameBankList->show();
    unsetCursor();
  }
  _ResizeRequired = true;
 
  if(listBoxBanks->count() > 0)
    listBoxBanks->setSelected(0, true);//select the first bank, even if not visible
  else{ //no bank, error
    setStatusText(tr("No bank data file could be found.")+ "<br>"
                  + tr("Searching for *moneypenny.bf files in:") + " " + 
                  banksDir.absPath() + "<br>"
                  + tr("Please check your medium!"),
                  red);
  }

  return listBoxBanks->count();
}

 /** Read Property of QString _BankName
 */
 QString GuiFindMedium::getBankName(){
   QString bankName = listBoxBanks->currentText();
   if(bankName == MONEYPENNY_ENCRYPTED_HBCI_CONF_FILE)
     bankName = ""; //keeps downwards compatibilty with MoneyPenny 1.1X, filename is moneypenny.bf
   else bankName = bankName + "_"; //filename is bankName_moneypenny.bf
   qDebug("Bankfile selected: %s", (bankName + MONEYPENNY_ENCRYPTED_HBCI_CONF_FILE).latin1());   
   return bankName;
 }
 
/** Sets the new status text and resizes and repaints the dialog
 */  
void GuiFindMedium::setStatusText(const QString & newText, const QColor & newColor){
     if(labelStatus->text()== newText)
      return;
      
     //change text. This may change the size of the label. 
     labelStatus->setText(newText);
     labelStatus->setPaletteBackgroundColor(newColor);
     update();
     qApp->processEvents(); //recalculate the dialog's layout for the new labelsize
 
     if(newColor == red)
       setCursor(Qt::waitCursor);      
     else unsetCursor();
     
     buttonOk->setEnabled(newColor == green);
     
     _ResizeRequired = true;
     repaint(); //now force resizing and repainting
}

/** Tries to write a testfile into dir. Returns true on success.
 */
bool GuiFindMedium::canWriteTestFile(){
       QString testFileName;
       bool result = false;
       if(_MountFlags.find(QString("rw"), 0) < 0){
         _IsWritable = false;
         qDebug("Device  %s is not mounted rw. Mountflags= %s",
         _MountPoint.latin1(), _MountFlags.latin1());
         return false;
       }
       testFileName = _MountPoint + "/moneypenny.tst";
       qDebug("Testfile: %s", testFileName.latin1());
       QFile testFile(testFileName);
       result = testFile.open(IO_WriteOnly); //do not rely on rw in /proc/mounts, try to write!
       system("sync"); //wait for all files to be written
       if(result) {      //an empty file has been written
         testFile.remove(); //remove it
         qDebug ("Wrote testfile: %s", testFileName.latin1()); 
       }  
       else qDebug("Device is mounted rw. Could not yet write testfile: %s", testFileName.latin1());
       return result;
 }

/** Searches for a Udrw-device. Uses a fixed path
*/
bool GuiFindMedium::searchForUdrwDevice(){
   bool result = false;
   qDebug("Using UD-RW features");
   _DeviceName = UDRW_DEVICE; // this is only sda1 (uba1) at this point
   qDebug("Device name: %s", _DeviceName.latin1());
   _Storage = STORAGE_USB;
   result = true;
   return result;
}

