/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUIFINDMEDIUM_H
#define GUIFINDMEDIUM_H

#include "guifindmediumbase.h"
#include "globals.h"

#include <qtimer.h>
/**
Searches for a freshly inserted removable storage medium by reading /proc/partitions
If it finds a suitable medium, it creates the moneypenny main directory, when accept is called. The absolute
path of this directory can be retrieved with getDirName. 
@author Walter Roth
*/
class GuiFindMedium : public GuiFindMediumBase
{
    Q_OBJECT
public:
    /**Constructor connects timer
    */
    GuiFindMedium(QWidget * parent);

    /**Destructor
    */
    virtual ~GuiFindMedium();
    
    /** Read property of StorageType m_Storage. 
    */
    const StorageType& GuiFindMedium::getStorage()const;
    
    /** Write property of StorageType _Storage. 
    */
    void GuiFindMedium::setStorage( const StorageType& _newVal);

    /** Read property of QString _CryptDirName. 
    */
    const QString& GuiFindMedium::getCryptDirName()const;
    
    /** Read property of QString _MountPoint. 
    */
    const QString& GuiFindMedium::getMountPoint()const;

    /** Read Property of QString _BankName
    */
    QString getBankName();

protected slots:    
    /** Searches for new partitions
    */
    void slotTimerDone();
    
    /** Tries to read from floppy or USB device
    */
    void slotSelectionChanged();

    /**
    *shows a short help message in a mesagebox.
    */
    void slotHelp();
    
    /** Closes dialog if _IsWriteable is true. Else repeats search.
    */
    void accept();

    /** Resets the flags and restarts timer
    */
    void slotTryAgain();
    
    /**Opens an Xterm and a file dialog
    */
    void slotSearchManually();
    
    /**Shows the extra buttons
    */
    void slotMore();
    
 private:
     /** Resizes window height, if required.
     */
     void paintEvent(QPaintEvent * pe);
     
     /** Starts the timer loop
     */
     void showEvent(QShowEvent * se);
     
     /** Stops the timer loop
     */
     void hideEvent(QHideEvent * se);

     /** Searches for a mounted floppy
     */
     bool searchForFloppy();
     
     /** Searches for a USB-device
     */
     bool searchForUsbDevice();
     /** Searches for a UDRW-device
     */
     bool searchForUdrwDevice();

     /** Searches for a new plugged in volume, 
      *  when moneypenny is running on mac osx
      */
     bool searchForVolumeOSX();

     /** Searches the mountpoint in mtab
     */
     QString findMountPoint();
    
     /** Read property of bool _IsWritable. 
     */
     const bool& GuiFindMedium::isWritable()const;
     
     /** Runs the specified process, spins the event loop and sete the _SkipTimer flag, so that timer events are not processed.
     */
     void runProcess(const QString & command);
     /** If more than one bank is available, shows the bank List.
     */
     int buildBankList();
     /** Sets the new status text and resizes and repaints the Widget
     */  
     void GuiFindMedium::setStatusText(const QString & newText, const QColor & newColor);     
     /** Tries to write a testfile into _MountPoint dir. Returns true on success
     */
     bool canWriteTestFile();
    
private:          

    /** Storage is the type of removable medium detected
    */
    StorageType _Storage;
    
    /** The timer for the check loop
    */
    QTimer _Timer;
    
    /**List of entries in /proc/partitions
    */
    QStringList _PartitionList;
    
    /** _CryptDirName is the directory name of the mounted medium
    */
    QString _CryptDirName;
    
    /** _CryptDirName is the directory name of the mounted medium
    */
    QString _MountPoint;

    /** _MountFlags are the flags as take from /proc/mounts
    */
    QString _MountFlags;
    /** _Device is the device name of the mounted medium (fd0, sda1)
    */
    QString _DeviceName;
 
     /**
     * Volume Mount path from a new found volume on osx
     */
    QString _OsxVolume;
    
    /**time until the dialog will close automatically
    */
    int _TimeToShow;  
    /** IsWritable is set according to the mount flags of the medium in /proc/mounts
    */
    bool _IsWritable;
    /**mkdir started, do not start again
    */
    bool _MkDirStarted;

   /**Flag to keep dialog open while searching manually
   */
   bool _SkipTimer;
 
   /** Flag to stop searching after medium has been found.
   */
   bool _MediumFound;  
   /**Flag for single repositioning. Blocks repeated repositioning in paintEvent
   */
   bool _ResizeRequired;
   /** Nmber of bank files found 
   */
   int _NumberOfBanks;
   /**Number of tries to write testfile
   */
   int _WriteCycles;
 };

#endif
