/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guihardwareconfig.h"
#include <qstring.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qfileinfo.h>


/**
*this is the constructor of the hardware configuration class
*
*@param main 
*@param parent the parent dialog of this widget
*@param name widgets title
*@param hobit
*/
GuiHardwareConfig::GuiHardwareConfig(QWidget * parent, const char *name)
   :GuiHardwareConfigBase(parent, name)
{
   _Process = NULL;
}

void GuiHardwareConfig::modemconfig()
{ 
  if(_Process)
    delete _Process;
  _Process = new QProcess(parent());
  _Process->addArgument("kppp");
  _Process->start();
  qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
}

/** destructor*/
GuiHardwareConfig::~GuiHardwareConfig(){
  if(_Process)
    delete _Process;
  qDebug("Destructor GuiHardwareConfig finished");
}


void GuiHardwareConfig::isdnconfig()
{
  if(_Process)
    delete _Process;
 _Process = new QProcess(parent());
 _Process->addArgument("sudo");
 _Process->addArgument("isdntool");
 _Process->start();
 qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
}

void GuiHardwareConfig::dslconfig()
{
  if(_Process)
    delete _Process;
 _Process = new QProcess(parent());
   _Process->addArgument("sudo");
   _Process->addArgument("/usr/sbin/pppoeconf");
 _Process->start();
 qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
}

void GuiHardwareConfig::lanconfig()
{
  if(_Process)
    delete _Process;
  _Process = new QProcess(parent());
  _Process->addArgument("sudo");
  _Process->addArgument("/usr/sbin/netcardconfig");
  _Process->start();
  qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
}

void GuiHardwareConfig::printerautosetup()
{
  emit raiseWidget(PRINTER_AUTOSETUP_WIDGET);
}

void GuiHardwareConfig::printermanualsetup()
{
  emit raiseWidget(PRINTER_MANUALSETUP_WIDGET);
}

void GuiHardwareConfig::wlanconfig()
{
  if(_Process)
    delete _Process;
  _Process = new QProcess(parent());
  _Process->addArgument("sudo");
  _Process->addArgument("/usr/sbin/wlcardconfig");
  _Process->start();
  qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
}

void GuiHardwareConfig::gprsconfig()
{
  if(_Process)
    delete _Process;
  _Process = new QProcess(parent());
  _Process->addArgument("sudo");
  _Process->addArgument("gprsconnect");
  _Process->start();
  qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
}

void GuiHardwareConfig::saveconfig() //presently disabled, no button
{
  if(_Process)
    delete _Process;
  _Process = new QProcess(parent());
  _Process->addArgument("sudo");
  _Process->addArgument("/usr/sbin/saveconfig");
  _Process->start();
  qDebug("Running process: %s", _Process->arguments().join(" ").latin1());
}

void GuiHardwareConfig::accept()
{
  emit finished();
}

/** Copy the predefined kppprc file to ~/.kde/share/config
*/
void GuiHardwareConfig::slotTryPredefinedModem(){
  QString destPath= "~/.kde/share/config";
  QString srcPath = MP_DIR() + MONEYPENNY_INTERNET_CONFIG_DIR;
  QString fileName ( "kppprc");
  bool ok = true;
  qDebug ("GuiHardwareConfig::slotPredefinedModem started");
  
  QDir dir(destPath);
  if(!dir.exists()){
    QString mkdirCommand = QString("mkdir -p " + destPath);    
    system(mkdirCommand.latin1());
  }  
  ok = copyConfigFile(srcPath, destPath, fileName, false); //copy as user knoppix
  
  if(!ok)
   QMessageBox::critical(this, tr("Error"), tr("Could not copy modem configuration file."));
  else
   QMessageBox::information(this, tr("Internet Configuration"), 
   tr("The predefined modem configuration file has been activated.\n You may now try to connect to the net with your modem attached to COM1 (= /dev/tty0).")); 
  modemconfig();
}
/** Copy the predefined isdn file to /etc/sysconfig/isdn
*/
void GuiHardwareConfig::slotTryPredefinedISDN(){
  QString destPath= "/etc/sysconfig";
  QString srcPath= MP_DIR() + MONEYPENNY_INTERNET_CONFIG_DIR;
  QString fileName ("isdn");
  bool ok = true;
  qDebug ("GuiHardwareConfig::slotPredefinedISDN started");
 
  ok = copyConfigFile(srcPath, destPath, fileName, true); //copy isdn config files as root
  if (ok){
   fileName = "device.ippp0";
   destPath = "/etc/isdn"; //copy the files for /etc/isdn
   ok = copyConfigFile(srcPath, destPath, fileName, true);   
  }
  if (ok){
   fileName = "device.ippp0+1";
   ok = copyConfigFile(srcPath, destPath, fileName, true);  
  }
  if (ok){
   fileName = "device.ippp0+1";
   ok = copyConfigFile(srcPath, destPath, fileName, true);  
  }
  if (ok){
   fileName = "ipppd.ippp0";
   ok = copyConfigFile(srcPath, destPath, fileName, true);  
  }
  if (ok){
   fileName = "ipppd.ippp0+1";
   ok = copyConfigFile(srcPath, destPath, fileName, true);  
  }   

  if(!ok){
   QMessageBox::critical(this, tr("Error"), tr("Could not copy isdn configuration files. Please configure manually."));
   isdnconfig();
  } 
  else{
   ok = system("sudo /etc/init.d/isdnutils start")==0; 
   if(ok){
     QMessageBox::information(this, tr("Internet Configuration"), 
     tr("The predefined ISDN configuration has been copied and the ISDN system has been started.") + "<br>" +
     tr("You may now try to connect to the internet via ISDN.")); 
     qDebug ("GuiHardwareConfig::slotPredefinedISDN: startup OK");
   }
   else{
      qDebug ("GuiHardwareConfig::slotPredefinedISDN: Starting ISDN system failed.");
      QMessageBox::warning(this, tr("Error"), 
      tr("Starting the ISDN system with the predefined configuration failed.") + "<br>" +
      tr("Most probably your hardware could not be detected automatically.") + "<br>" +
      tr("Please configure the ISDN system manually.")); 
      qDebug ("GuiHardwareConfig::slotPredefinedISDN: startup failed");
      isdnconfig();
   }
  } 
}

/** Copies file from src to dest using root privileges. For use with KNOPPIX only.
 * Uses sudoe if asRoot is true.
 */
bool GuiHardwareConfig::copyConfigFile(QString srcPath, QString destPath, QString fileName, bool asRoot)
{ 
 QString fileSrcPath = srcPath + "/" + fileName;
 QFileInfo fileInfo (fileSrcPath);
 
 bool ok = fileInfo.exists();
 if (ok){
   QString cpCommand = ("cp " + fileSrcPath + " " + destPath);
   if( asRoot )
     cpCommand = "sudo " + cpCommand;
     
   ok = system(cpCommand.latin1()) == 0;
   
   if(ok)    
     qDebug ("GuiHardwareConfig::CopyConfigFile: copy command ok: %s", cpCommand.latin1());
   else 
     qDebug ("GuiHardwareConfig::CopyConfigFile: copy command failed: %s", cpCommand.latin1());
 }
 else qDebug("GuiHardwareConfig::CopyConfigFile: File not found: %s", fileSrcPath.latin1());
 return ok;
}

