/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guilogin.h"

#include <qcombobox.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qmovie.h>
#include <qlabel.h>
#include <qstring.h>
#include <string.h>
#include <qlineedit.h>


/**
*constructor of GuiLogin
*
*@param parent a pointer on the main widgets widget stack
*@param stack main QWidget Stack in the 'main frame'
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiLogin::GuiLogin(QWidget* parent, GuiMain * guimain, const char *name, HoBIT *hobit)
    :GuiLoginBase(parent, name)
{
  _hobit = hobit;
  _guimain = guimain;


  QFont font = comboBoxAccounts->font();
  //font.setBold(true);
  font.setPixelSize(14);
  comboBoxAccounts->setFont(font);
  buttonEdit->hide();
  refresh();

}

/** destructor*/
GuiLogin::~GuiLogin()
{
  qDebug("Destructor GuiLogin finished");
}


/**
*slot for the login
*/
void GuiLogin::slotLogin()
{
  QString account = comboBoxAccounts->currentText();
  double accountNumber = account.toDouble();
  if(account.isEmpty() || accountNumber == 0.0)
    QMessageBox::critical(this, tr("Error"),tr("No account has been selected. Please select an account from the account list."));
  else _guimain->setCurrentAccount(account);
}

/**
*slot for the closing
*/
void GuiLogin::slotClose()
{
  _hobit->removeUncryptedEnvironment();
  _guimain->close(); //bypass the normal close message in slotClose
}

/**
*slot changing the description of the account
*/
void GuiLogin::slotShowDescription()
{
  QString description;
  Pointer<Account> account = _hobit->findAccount(comboBoxAccounts->currentText().ascii());

  if (account.isValid())
  {
    description = QString(account.ref().accountName().c_str());
    if(description.isEmpty())
    {
      description = tr("Description of Account");
      lineEditDescription->setText(description);
      buttonEdit->show();
    }
    else
    {
      lineEditDescription->setText(QString(account.ref().accountName().c_str()));
      buttonEdit->hide();
    }
  }
  else //no valid account
  {
    lineEditDescription->setText(tr("No account selected"));
    buttonEdit->hide();
  }
}


/**
*function for refreshing
*/
void GuiLogin::refresh()
{

  comboBoxAccounts->clear();

  list<Pointer<Account> > accounts = _hobit->listAccounts();
  list<Pointer<Account> >::iterator  accit;

  if (accounts.empty())
  {
    comboBoxAccounts->insertItem(tr("No accounts found yet"));
    buttonLogin->hide();
    _guimain->slotRaiseWidget(HBCI_WIZARD);
  }
  else
  {
    int nAccounts = accounts.size();
    if(nAccounts > 1)
      comboBoxAccounts->insertItem(QString(tr("Account List")));
    qDebug("Number of accounts: %i", nAccounts );  	
    for (accit=accounts.begin(); accit!=accounts.end(); accit++)
    {
      string accountId = (*accit).ref().accountId();
      comboBoxAccounts->insertItem( QString(accountId.c_str()) );
      buttonLogin->show();
    }
    slotShowDescription();
  }
}

/** Shows the edit button, if the user starts typing text in description edit field.
*/
void GuiLogin::slotShowEditButton()
{
  if(!buttonEdit->isVisible())
    buttonEdit->show();
}

/** Sets the description of the account to text in lineEditDescription
*/
void GuiLogin::slotNewDescription()
{
  QString newDescription = lineEditDescription->text();
  qDebug("New Description: %s", newDescription.latin1());
  _hobit->changeAccountName(comboBoxAccounts->currentText().latin1(), newDescription.latin1());
  buttonEdit->hide();
}

