/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
  
  #include "guishowdata.h"
  #include "hobit.h"
  
  #include <qlistview.h>
  #include <qstring.h>


  
/** Constructor
*/
GuiShowData::GuiShowData(QWidget *parent, const char *name, HoBIT * hobit)
 : GuiShowDataBase(parent, name)
{
 qDebug("Constructor GuiShowData");
 _hobit = hobit;
}

/**Destructor
*/
GuiShowData::~GuiShowData()
{
 qDebug("Destructor GuiShowData");
}

/** set hobit object*/
void GuiShowData::setHoBIT(HoBIT* hobit){
 _hobit = hobit;
 refresh();
}

/** Emits signal back.
*/
void GuiShowData::slotBack(){
 emit back();
}

/** Updates the data listview.
*/
void GuiShowData::refresh(){
 updateList();
}

/**
* updates the list of users
*/
void GuiShowData::updateList()
{
  listViewConnections->clear();
  Pointer<User>  user;
  list<Pointer<User> > users = _hobit->listUsers();
  Pointer<Bank> bank;
  Pointer<Medium> medium;


  if (!users.empty())
  {
    user = (*users.begin());

    bank =(user).ref().bank();
    medium =(user).ref().medium();
    QString bankname = bank.ref().bankCode().c_str();
    QString version, versions, num;

    // get used version
    num.setNum(bank.ref().hbciVersion());
    version = num;

    // get accepted versions
    list <int> vlist = bank.ref().hbciVersions();
    list <int>::iterator vit;
    for (vit = vlist.begin(); vit!= vlist.end(); vit++)
    {
      num.setNum(*vit);
      versions+= num;
      versions+= "      ";
    }

  listViewConnections->setColumnWidthMode(0, QListView::Maximum);
  listViewConnections->setColumnWidthMode(1, QListView::Maximum);

    new QListViewItem(listViewConnections, tr("Bank:\t"), bank.ref().name().c_str());
    new QListViewItem(listViewConnections, tr("Bankcode:\t"), bank.ref().bankCode().c_str());
    new QListViewItem(listViewConnections, tr("UserId:\t"), user.ref().userId().c_str());
    new QListViewItem(listViewConnections, tr("Medium type for key:\t"), medium.ref().mediumTypeName().c_str());
    new QListViewItem(listViewConnections, tr("SystemId:\t"), user.ref().systemId().c_str());
    new QListViewItem(listViewConnections, tr("Medium path:\t"), medium.ref().mediumName().c_str());
    new QListViewItem(listViewConnections, tr("Used HBCI-Version:\t"), version);
    new QListViewItem(listViewConnections, tr("Supported HBCI-Versions:\t"), versions);

  } // end if

  else
    new QListViewItem(listViewConnections, "State    ", "No HBCI-Configuration found");
    
}
/** Emits signal help request.
*/
void GuiShowData::slotHelp(){
 emit helpRequest("showData.html");
}

