/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "hobit_bank.h"

#include <qstringlist.h>
#include <qfile.h>
#include <qtextstream.h>

HoBIT_Bank::HoBIT_Bank()
{
}


HoBIT_Bank::~HoBIT_Bank()
{
}


/** Sets the memberfields from a single line of the .csv file.
*/
bool HoBIT_Bank::setFromCsvLine(const QString  & csvLine){
   QStringList row = QStringList::split( ";", csvLine );
   if(row.count() < 9){ //invalid Line
  //  qDebug("Invalid bank record in list."); //Thanks to Susanne Haubner there are too many of these!
    return false;
   }
   if(row[1].length() == 8){  //BankId (BLZ) must have exactly 8 digits
     _BankId = row[1];
     _BankName = row[2];
     _Location = row[3]; 
     if(row[6].length() > 0)
       _Server = row[6];  //Url
     else if(row[7].length() > 0)
      _Server = row[7];  //IP address
     else _Server = QObject::tr("No HBCI server. HBCI-PIN-TAN only?"); 
      QString HBCIVersion = row[8];
     _HBCIVersion = convertHBCIVersionString(HBCIVersion);
     return true;
   }
   else return false;
}

/** Read property of QString _BankName. 
*/
const QString& HoBIT_Bank::getBankName()const{
	return _BankName;
}

/** Write property of QString _BankName. 
*/
void HoBIT_Bank::setBankName( const QString& _newVal){
	_BankName = _newVal;
}

/** Read property of QString _BankId. 
*/
const QString& HoBIT_Bank::getBankId()const{
	return _BankId;
}

/** Write property of QString _BankId. 
*/
void HoBIT_Bank::setBankId( const QString& _newVal){
	_BankId = _newVal;
}


/** Read property of QString _Location. 
*/
const QString& HoBIT_Bank::getLocation()const{
	return _Location;
}

/** Write property of QString _Location. 
*/
void HoBIT_Bank::setLocation( const QString& _newVal){
	_Location = _newVal;
}


/** Read property of QString _Server. 
*/
const QString& HoBIT_Bank::getServer()const{
	return _Server;
}

/** Write property of QString _Server. 
*/
void HoBIT_Bank::setServer( const QString& _newVal){
	_Server = _newVal;
}

/** Reads a line from bankdata.txt
*/
bool HoBIT_Bank::setFromTxtLine(const QString & txtLine){
   QStringList row = QStringList::split( "\t", txtLine );
   if(row[0].length() == 8){  //BankId (BLZ) must have exactly 8 digits
     _BankId = row[0];
     _BankName = row[2];
     _Location = row[3];    
     _Server = "unknown";  //Url is not in list
     _HBCIVersion = 201; // not in txt list
     return true;
   }
   else return false;
}
/** Returns the HBCI version of the server
*/
int HoBIT_Bank::getHBCIVersion(){
    return _HBCIVersion;
}
/** Set the HBCI Version for the server
*/
void HoBIT_Bank::setHBCIVersion(int _newVal){
  _HBCIVersion = _newVal;
}
/** Converts a version string to the corresponding int. 
*/
int HoBIT_Bank::convertHBCIVersionString(const QString & version){
    QString ver = version; 
     if(version.isEmpty())
        ver= "2.0.1"; 
     if(ver.find("2.0.1") >= 0)  
       return 201;
     else if(ver.find("2.1") >= 0)  
       return 210;
     else if(ver.find("2.2") >= 0)  
       return 220; //re-activated 18.10.2005 WR //limit HBCI version to 2.1. There was trouble with 2.2 when getting the account list.
     else return 201; //take the lowest for safety  
}
/** Returns  HBCI version as a string
*/
QString  HoBIT_Bank::getHBCIVersionString(){
 QString result;
 switch (_HBCIVersion){
   case 201 : result = QString("2.0.1"); break;
   case 210 : result = QString("2.1");   break;
   case 220 : result = QString("2.2");   break;
   default : result = QString("2.0.1");  break;
 } 
 return result;
}

/** Returns true, if the specified job is supported by the bank. 
* The job string is the HBCI id (Kennung) for this job (HKTUE for a dated transfer).
* This function parses the openhbci.conf in moneypenny_data dir.
*/
bool HoBIT_Bank::supportsJob(const QString confFileName, const QString & job){
 QFile confFile(confFileName);
 if(!confFile.exists())
  return false;
 if(!confFile.open(IO_ReadOnly))
  return false;
 QTextStream s(&confFile);
 QString line;
 bool found = false;
 //search the config file for the job
 while(!s.atEnd() && (!found)){
   s >> line;
   found = line.find(job) >=0;
 } 
 confFile.close();
 if(!found)
  qDebug("HoBIT_Bank::supportsJob: Bank does not support the following job type: %s", job.latin1());
 return found;
}

/** Writes the bank to a binary QDataStream.
*/
bool HoBIT_Bank::toDataStream(QDataStream & s){
 try{
      /** BankName is the (not unique) name of the bank
          */
      s <<_BankName;
          /** BankId is the unique identifier of the bank (BLZ for a German bank)
          */
      s <<_BankId;    
          
          /** Location is the city where the bank is
          */
      s <<_Location;
          /** Server is the URL or IP address of the bank's HBCI server
          */
      s <<_Server;
          /**HBCI-Version supported by server
          */
      s <<  ( Q_INT32 ) _HBCIVersion;
   }
   catch(...){
      return false;
   }
 return true;  
}

/** Reads the bank from a binary QDataStream.
*/
bool HoBIT_Bank::setFromDataStream(QDataStream & s){
 try{
     /** BankName is the (not unique) name of the bank
         */
     s >>_BankName;
         /** BankId is the unique identifier of the bank (BLZ for a German bank)
         */
     s >>_BankId;    
         
         /** Location is the city where the bank is
         */
     s >>_Location;
         /** Server is the URL or IP address of the bank's HBCI server
         */
     s >>_Server;
         /**HBCI-Version supported by server
         */
     Q_INT32 version;
     s >> version;
     _HBCIVersion = version;
 }  
 catch(...){
    return false;
 }
 
 return true;  
}
