/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HOBIT_BANK_H
#define HOBIT_BANK_H

#include <qlistview.h>

/**
Contains the data read from the fints_institute.csv list by Zentraler Kreditauschuss
file is aavilable from www.hbci-zka.de. This file contains only banks supporting HBCI.
Use setFRomCsvLine to evaluate a line.
May also be used with the bankadata.txt list of the Deutsche Bundesbank, which contains all German banks.
Use setFromTxtLine to evaluate a line.
@author Walter Roth
*/
class HoBIT_Bank 
{
public:
    HoBIT_Bank();

    ~HoBIT_Bank();

    /** Sets the memberfields from a single line of the fints_institute.csv file.
    */
    bool setFromCsvLine(const QString  & csvLine);
    /** Reads a line from bankdata.txt
    */
    bool setFromTxtLine(const QString & txtLine);

   /** Read property of QString BankId. 
   */
   const QString& HoBIT_Bank::getBankId()const;
   /** Write property of QString BankId. 
   */
   void HoBIT_Bank::setBankId( const QString& _newVal);
    /** Read property of QString _BankName. 
   */
   const QString& HoBIT_Bank::getBankName()const;
   /** Write property of QString _BankName. 
   */
   void HoBIT_Bank::setBankName( const QString& _newVal);
   /** Read property of QString _Location. 
   */
   const QString& HoBIT_Bank::getLocation()const;
   /** Write property of QString _Location. 
   */
   void HoBIT_Bank::setLocation( const QString& _newVal);
   /** Read property of QString _Server. 
   */
   const QString& HoBIT_Bank::getServer()const;
   /** Write property of QString _Server. 
   */
   void HoBIT_Bank::setServer( const QString& _newVal);
   /** Returns the HBCI version of the server
   */
   int getHBCIVersion();

   /** Set the HBCI Version for the server
   */
   void setHBCIVersion(int _NewVal);

   /** Converts a version string to the corresponding int. 
   */
   static int convertHBCIVersionString(const QString & version);
   /** Returns  HBCI version as a string
   */
   QString getHBCIVersionString();
   /** Writes the bank to a binary QDataStream.
   */
   bool toDataStream(QDataStream & s);
   /** Reads the bank from a binary QDataStream.
   */
   bool setFromDataStream(QDataStream & s);

   /** Returns true, if the specified job is supported by the bank. 
     * The job string is the HBCI id (Kennung) for this job (HKTUE for a dated transfer).
     * This function parses the openhbci.conf in moneypenny_data dir.
     * TODO : get the info from elsewhere
     */
   static bool supportsJob(const QString confFileName, const QString & job);
   
private:

    /** BankName is the (not unique) name of the bank
    */
    QString _BankName;
    /** BankId is the unique identifier of the bank (BLZ for a German bank)
    */
    QString _BankId;    
    
    /** Location is the city where the bank is
    */
    QString _Location;
    /** Server is the URL or IP address of the bank's HBCI server
    */
    QString _Server;
    /**HBCI-Version supported by server
    */
    int _HBCIVersion;

};

#endif
